﻿using Microsoft.Live;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MyPhotoAlbum.Data
{
    public class OneDriveDataProvider : INotifyPropertyChanged
    {
        private LiveAuthClient _authClient;
        private LiveConnectSession _session;

        private string _userName;
        public string UserName
        {
            get { return this._userName; }
            set
            {
                _userName = value;
                NotifyPropertyChanged("UserName");
            }
        }

        public OneDriveDataProvider()
        {
            //InitProvider();
        }

        // Initialize the user's session, request access if 1st time, and attach to appropriate events
        private async Task<bool> InitProvider()
        {
            if (!Windows.ApplicationModel.DesignMode.DesignModeEnabled)
            {
                LiveAuthClient authClient = new LiveAuthClient();
                LiveLoginResult authResult = await authClient.LoginAsync(new List<string>() { "wl.signin", "wl.basic", "wl.skydrive_update" });
                if (authResult.Status == LiveConnectSessionStatus.Connected)
                {
                    // Add code to handle session held in the Session property
                    _session = authResult.Session;

                }
                await LoadProfile();
            }

            return true;
        }

        // Load the user's profile information
        private async Task<bool> LoadProfile()
        {
            LiveConnectClient client = new LiveConnectClient(_session);
            LiveOperationResult liveOpResult = await client.GetAsync("me");
            dynamic dynResult = liveOpResult.Result;
            UserName = dynResult.name;

            return true;
        }

        public async Task<object> TestData()
        {
            LiveConnectClient client = new LiveConnectClient(_session);
            LiveOperationResult liveOpResult = await client.GetAsync("me/skydrive");
            dynamic dynResult = liveOpResult.Result;

            return dynResult;
        }

        public async Task<object> GetObject(string path)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            if (String.IsNullOrEmpty(path))
            {
                path = "me/skydrive";
            }


            LiveConnectClient client = new LiveConnectClient(_session);
            LiveOperationResult liveOpResult = await client.GetAsync(path);
            dynamic dynResult = liveOpResult.Result;

            return dynResult;
        }

        public async Task<List<object>> GetFolderItems(string path, string filter)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            if (String.IsNullOrEmpty(path))
            {
                path = "me/skydrive";
            }

            if (!String.IsNullOrEmpty(filter))
            {
                filter = "?filter=" + filter;
            }

            LiveConnectClient client = new LiveConnectClient(_session);
            LiveOperationResult liveOpResult = await client.GetAsync(path + "/files" + filter);
            dynamic dynResult = liveOpResult.Result;

            return new List<object>(dynResult.data);
        }

        public async Task<bool> CreateFolder(string path, string name)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            if (String.IsNullOrEmpty(path))
            {
                path = "me/skydrive";
            }

            try
            {
                var folderData = new Dictionary<string, object>();
                folderData.Add("name", name);

                LiveConnectClient liveClient = new LiveConnectClient(_session);
                LiveOperationResult operationResult =
                    await liveClient.PostAsync(path, folderData);
                dynamic result = operationResult.Result;
                return true;
            }
            catch (LiveConnectException exception)
            {
                return false;
            }
        }

        public async Task<bool> DeleteFolder(string path)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            if (String.IsNullOrEmpty(path))
            {
                path = "me/skydrive";
            }

            try
            {
                LiveConnectClient liveClient = new LiveConnectClient(_session);
                LiveOperationResult operationResult = await liveClient.DeleteAsync(path);

                return true;
            }
            catch (LiveConnectException exception)
            {
                return false;
            }
        }

        public async Task<bool> CopyObject(string path, string destination)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            try
            {
                LiveConnectClient liveClient = new LiveConnectClient(_session);
                LiveOperationResult operationResult = await liveClient.CopyAsync(path, destination);

                return true;
            }
            catch (LiveConnectException exception)
            {
                return false;
            }
        }

        public async Task<bool> DownloadObject(string path)
        {
            if (_session == null)
            {
                await InitProvider();
            }

            try
            {
                LiveConnectClient liveClient = new LiveConnectClient(_session);
                LiveDownloadOperation operation = await liveClient.CreateBackgroundDownloadAsync(path);
                var result = await operation.StartAsync();
            }
            catch
            {
                return false;
            }
        }
        // Declare the PropertyChanged event.
        public event PropertyChangedEventHandler PropertyChanged;

        // NotifyPropertyChanged will raise the PropertyChanged event, 
        // passing the source property that is being updated.
        public void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this,
                    new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
